/**
 * @(#) Entity.java
 */

package org.omg.tacsit.controller;

import org.omg.tacsit.geometry.GeodeticPosition;

/**
 * The {@linkplain Entity} class is an abstraction of the tactical objects that can be
 * displayed on a Tactical Situation display (TacSit).
 * <p>
 * Entities are modeled such that it is possible for clients to establish there
 * specific type and geometry and hence proceed to interact with them in an
 * application specific manner.
 * <p>
 * Entities need to be able to indicate if they are still valid: even though
 * they may have been part of a selection at a certain point in time, by the
 * time that the entities of that selection are accessed the entities may not
 * exist any more in reality.
 */
public interface Entity
{
	/**
	 * Checks to see if this {@linkplain Entity} is equal to another.
	 * @return true if the given Entity corresponds to this Entity (i.e. the
	 * Tactical Objects are the same), false otherwise.
	 */
	boolean equals( Entity entity );
	
	/**
	 * Checks to see whether or not this {@linkplain Entity} is still valid.
	 * 
	 * @return true if this instance is valid
	 */
	boolean isValid( );
	
	/**
	 * Gets the type of this {@linkplain Entity}.
	 * 
	 * @return the type of the {@linkplain Entity}. This corresponds to the application
	 * specific class of the tactical object that has been added to the TacSit.
	 */
	EntityType getType( );
	
	/**
	 * Checks to see whether or not this {@linkplain Entity} can be described by
   * a single point.
   * <p>
   * This is a convenience method for indicating whether the entity is to be
	 * handled as though it is a single point rather than a complex geometry.
	 * This is useful for containment and other queries.
	 * 
	 * @return true if this Entity can be entirely described by a point.
	 */
	boolean isPointEntity( );
	
  /**
   * Gets the reference point for this {@linkplain Entity}.  For a Point 
   * {@linkplain Entity}, this would usually be its exact location.  For a
   * shape {@linkplain Entity}, this might be its center point.
   * @return the {@link GeodeticPosition} to be used as the main point of 
   * reference for this entity.
   */
	GeodeticPosition getReferencePoint();
}
