/**
 * @(#) SelectionManager.java
 */

package org.omg.tacsit.controller;

import java.util.Collection;
import java.util.List;

/**
 * The {@linkplain SelectionManager} is the entry point for managing the 
 * selections of the {@link TacsitController}.
 * <p>
 * Multiple selections can be managed using the {@linkplain SelectionManager}. Each
 * selection has a specific {@link SelectionType}. {@linkplain SelectionType}s are 
 * predefined. Each selection is a list of the type {@link Entity}. This means that 
 * each  element can only occur once in the selection and the order of the elements 
 * in the selection is not determined.
 * <p>
 * Managing a selection through the {@link SelectionManager} means that the 
 * {@linkplain SelectionManager} allows retrieving and modifying each of the 
 * selections. It also allows users to register and unregister selection listeners 
 * which will be notified in case of a change in a selection.
 * <p>
 * The scope of the {@link SelectionManager} is determined by the 
 * {@link SelectionMethodology} of the {@link TacsitController}.
 */
public interface SelectionManager
{
	/**
	 * Registers the given {@link SelectionListener} to this 
   * {@linkplain SelectionManager}.
	 * <p>
	 * After registration the {@link SelectionListener} will be notified through its
	 * only method each time that a change occurs in one of the selections
	 * managed by this {@linkplain SelectionManager}. Registering a 
   * {@linkplain SelectionListener} that was already registered with this 
   * {@linkplain SelectionManager} therefore does not have any effect.
	 * 
   * @param listener The listener that should receive notifications about
   * selection changes.
	 */
	void addSelectionListener( SelectionListener listener );
	
	/**
	 * Unregisters the given {@link SelectionListener} from this 
   * {@linkplain SelectionManager}.
	 * <p>
	 * Afterwards the {@link SelectionListener} will not be notified of 
   * changes in the  selections of this {@linkplain SelectionManager}. 
   * Unregistering a {@linkplain SelectionListener} that was not registered 
   * therefore does not have any effect.
	 * 
   * @param listener The listener that should no longer receive notifications.
	 */
	void removeSelectionListener( SelectionListener listener );
	
	/**
	 * Removes all Entities from the given {@link Entity} Set from the 
   * selection from the selection to which they belong.
	 * <p>
	 * Afterwards, these entities will not be contained in any selection until
	 * added (where contains is based on equals).
	 * <p>
	 * All {@link SelectionListener}s registered at the time of the 
   * removeFromSelection are notified by a call of their method 
   * selectionChanged for each {@link SelectionType} from which at least one 
   * {@linkplain Entity} was removed.
	 * 
   * @param The {@link Entity} objects which should be removed from the selection.
	 */
	void removeFromSelection(Collection<Entity> entities );
	
	/**
	 * Clears all selections of this {@link SelectionManager}. Afterwards for any
	 * {@link SelectionType}, the selection of that type contains no Entities.
	 * <p>
	 * All {@link SelectionListener}s registered at the time of the clearAllSelections
	 * are notified by a call of their method selectionChanged.
	 */
	void clearAllSelections( );
	
	/**
	 * Changes the selection of the given {@link SelectionType} to the given 
   * {@link Entity} Set.  The given {@linkplain Entity} Set should not be 
   * empty (see clearSelection and clearAllSelections).
	 * <p>
	 * Afterwards the set of Entities of getSelection( type ) equals the set of
	 * Entities of the given EntitySet.
	 * <p>
	 * All SelectionListeners registered at the time of the setSelection are
	 * notified by a call of their method selectionChanged.
	 * 
   * @param entities The {@link Entity} set that should be the new selection.
   * @param type The type of selection being changed.
	 */
	void setSelection(Collection<Entity> entities, SelectionType type );
	
	/**
	 * Returns the selection of the given SelectionType of this
	 * SelectionManager.
	 * <p>
	 * Note that editing the returned EntitySet does not have an impact on the
	 * selection of the given SelectionType of this SelectionManager.
	 * 
   * @param type The selection type of interest.
	 * @return The {@linkplain Entity} Set that is selected for the given type.
	 */
	List<Entity> getSelection( SelectionType type );
	
	/**
	 * Adds all elements of the given array of Entities to the selection of the
	 * given SelectionType of this SelectionManager. The given EntityList should
	 * not be empty.
	 * <p>
	 * Afterwards, getSelection( type ) contains all elements of the given array
	 * of Entities as well as any entities previously selected for that type.
	 * <p>
	 * Each Entity can be part of only one selection at a time. Therefore, all
	 * entities of the given array of Entities that were part of a selection of
	 * another SelectionType than the given type are removed from those
	 * selections.
	 * <p>
	 * All SelectionListeners registered at the time of the addToSelection are
	 * notified by a call of their method selectionChanged.
	 * 
   * @param entities The {@link Entity} Set that should be added to the 
   * selection.
   * @param type The type of selection being added to.
	 */
	void addToSelection(Collection<Entity> entities, SelectionType type );
	
	/**
	 * Clears the selection of the given SelectionType of this SelectionManager.
	 * Afterwards the selection of the given SelectionType of this
	 * SelectionManager does not contain any Entities. All other selections of
	 * this SelectionManager are unchanged.
	 * <p>
	 * All SelectionListeners registered at the time of the clearSelection are
	 * notified by a call of their method selectionChanged.
	 * 
   * @param The type of selection being cleared.
	 */
	void clearSelection( SelectionType type );
	
	
}
