/**
 * @(#) Viewport.java
 */

package org.omg.tacsit.controller;

import java.awt.Point;
import java.util.Collection;
import org.omg.tacsit.geometry.Distance;
import org.omg.tacsit.geometry.GeodeticPosition;

/**
 * The Viewport class enables a TacSit client to interact with the basic
 * properties of a TacSit view. In particular the view's name, where it is
 * centered, how it is scaled, projected and oriented, and the selections it has
 * (through its SelectionManager instance).
 */
public interface Viewport
{
	/**
	 * Returns the name of the {@linkplain Viewport}
	 * 
	 * @return The name of the {@linkplain Viewport}
	 */
	String getName( );
	
	/**
	 * Set the ViewEye properties of the Viewport to the given
	 * ViewEyeProperties.
	 * <p>
	 * If any of the attributes of the given ViewEyeProperties object are not
	 * set, the current value of the Viewport for that attribute will not be
	 * affected.
	 * <p>
	 * Later changes to the given ViewEyeProperties object do not have an effect
	 * on the Viewport.
	 * 
   * @param viewEyeProps The new view eye to orient the {@linkplain Viewport}
   * on.
	 */
	void setViewEye( ViewEyeProperties viewEyeProps );
	
	/**
	 * Returns a copy of the current ViewEyeProperties of the Viewport.
	 * Modification to the returned ViewEyeProperties object will have no effect
	 * on the Viewport. To effect change, call the setViewEye method with the
	 * desired properties.
	 * 
	 * @return The current view eye that the {@linkplain Viewport} is
   * oriented on.
	 */
	ViewEyeProperties getViewEye( );
	
	/**
	 * Registers the given ViewportChangeListener to this Viewport.
	 * <p>
	 * After registration the ViewportChangeListener will be notified through
	 * its only method each time that a viewport is changed.
	 * <p>
	 * Registering a ViewportChangeListener that was already registered with
	 * this Viewport does not have any effect.
	 * 
   * @param listener The listener that wants to receive notification about
   * changes to the {@linkplain Viewport}.
	 */
	void addViewportChangeListener( ViewportChangeListener listener );
	
	/**
	 * Unregisters the givenViewportChangeListener from this Viewport.
	 * <p>
	 * Afterwards the ViewportChangeListener will not be notified of Viewport
	 * changes.
	 * <p>
	 * Unregistering a ViewportChangeListener that was not registered does not
	 * have any effect.
	 * 
   * @param listener The listener that no longer wants to be notified.
	 */
	void removeViewportChangeListener( ViewportChangeListener listener );
	
	/**
	 * Set the name of the {@linkplain Viewport} to the given name.
	 * 
   * @param name The new name of the {@linkplain Viewport}.
	 */
	void setName( String name );
	
	/**
	 * Converts the passed screen position into a geo position (pixels). Note
	 * that the conversion may not be valid.
	 * <p>
	 * These conversion methods are useful if you are trying to do anything
	 * special with OS GUI libraries (e.g. mouse controllers, popup windows, or
	 * other things of that sort).
	 * 
   * @param screenPos The position (in pixels) on the screen to convert to 
   * geodetic coordinates.
	 * @return The {@link GeodeticPosition} at the location on screen.
	 */
	GeodeticPosition convertScreenPosition( Point screenPos );
	
	/**
	 * Converts the given GeoPosition into a ScreenPosition (pixels). Note that
	 * the conversion may not be valid.
	 * <p>
	 * These conversion methods are useful if you are trying to do anything
	 * special with OS GUI libraries (e.g. mouse controllers, popup windows, or
	 * other things of that sort).
	 * 
   * @param geoPos The geodetic coordinates in the world to convert
	 * @return The position on the screen (in pixels) where the 
   * {@link GeodeticPosition} is located.
	 */
	Point convertGeoPosition( GeodeticPosition geoPos );
	
	/**
	 * Offset and scale the viewport to contain all points (as possible) passed
	 * in the given GeoPositions array. 
	 * @param points A group of points that should be made visible on the screen.
   * @param margin The additional space that needs to be visible around the 
   * broadest points in the points list.
	 */
	void scaleToPoints( Collection<GeodeticPosition> points, Distance margin );
	
	
}
