/**
 * @(#) Distance.java
 */
package org.omg.tacsit.geometry;

/**
 * A representation of the space between two points.
 * <p>
 * {@linkplain Distance} objects are immutable.  All methods that operate on the 
 * state of the {@linkplain Distance} do not modify its value.
 */
public interface Distance
{
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in feet.
   * @return The distance in feet.
   */
  double getFeet();
  
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in yards.
   * @return The distance in yards.
   */
  double getYards();
  
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in miles.
   * @return The distance in miles.
   */
  double getMiles();
  
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in meters.
   * @return The distance in meters.
   */
  double getMeters();
  
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in kilometers.
   * @return The distance in kilometers.
   */
  double getKilometers();
  
  /**
   * Gets the value of this {@linkplain Distance}, as expressed in nautical miles.
   * @return The distance in nautical miles.
   */
  double getNauticalMiles();
  
  /**
   * Creates a new {@linkplain Distance} which represents the sum of
   * this {@linkplain Distance} and the parameter (this + that).
   * <p>
   * Neither this distance nor the parameter are modified.
   * @param distance The {@linkplain Distance} to add.
   * @return A new {@linkplain Distance} which represents (this + that).
   */
  Distance add(Distance distance);
  
  /**
   * Creates a new distance which represents the difference of this 
   * {@linkplain Distance} and the parameter (this - that).
   * <p>
   * Neither this {@linkplain Distance} nor the parameter are modified.
   * @param distance The {@linkplain Distance} to subtract.
   * @return A new {@linkplain Distance} which represents (this - that).
   */
  Distance subtract(Distance distance);
  
  /**
   * Creates a new {@linkplain Distance} which represents this 
   * {@linkplain Distance} times a scalar value (this * scalarValue).
   * This {@linkplain Distance} is not modified.
   * <p>
   * A typical usage of this would be in a formula that requires a {@linkplain Distance}; 
   * for instance, the circumference of a circle = 2 * Pi * radius.  In this case, the
   * radius is a {@linkplain Distance}, the scalar value being multiplied is (2 * Pi), 
   * and the return value is the circumference.
   * <p>
   * Note: No option is given to multiplyBy({@linkplain Distance}), because the logical
   * result of such a function would be an Area value, not a {@linkplain Distance}.
   * @param scalarValue The scalar value to multiply this {@linkplain Distance} by.
   * @return A new {@linkplain Distance} which represents (this * scalarValue).
   */
  Distance multipliedBy(double scalarValue);
  
  /**
   * Creates a new {@linkplain Distance} which represents this 
   * {@linkplain Distance} divided by a scalar value (this / scalarValue).
   * This {@linkplain Distance} is not modified.  
   * <p>
   * A typical usage of this would be in a formula that requires a {@linkplain Distance}; 
   * for instance, the computing half the distance distance traveled.  In this case, the
   * total distance is a {@linkplain Distance}, the scalar value being divided is 2, and 
   * the return value is the half the total distance.
   * @param scalarValue The scalar value to divide this {@linkplain Distance} by.
   * @return A new {@linkplain Distance} which represents (this / scalarValue).
   */
  Distance dividedBy(double scalarValue);
  
  /**
   * Creates a new {@linkplain Distance} that represents this 
   * {@linkplain Distance} by another distance (this / that).
   * This {@linkplain Distance} is not modified.
   * <p>
   * Note: When dividing a unit of measurement by the same unit of measurement, the
   * units cancel each other out, and the resulting value is scalar.
   * @param distance The {@linkplain Distance} to divide this {@linkplain Distance} by.
   * @return A scalar value which represents (this / that).
   */
  double dividedBy(Distance distance);
}
